@@ -141,9 +141,6 @@ function enable() {
     Workspace.Workspace.prototype._calculateWindowTransformationsNatural = function(clones) {
         // As we are using pseudo-random movement (See "slot") we need to make sure the list
         // is always sorted the same way no matter which window is currently active.
-        clones = clones.sort(function (win1, win2) {
-            return win2.metaWindow.get_stable_sequence() - win1.metaWindow.get_stable_sequence();
-        });
 
         let node = this.actor.get_theme_node();
         let columnSpacing = node.get_length('-horizontal-spacing');
@@ -319,94 +316,22 @@ function enable() {
         let buttonOuterHeight, captionHeight;
         let buttonOuterWidth = 0;
 
-        let targets = [];
+        let slots = [];
         for (let i = 0; i < rects.length; i++) {
             rects[i].x = rects[i].x * scale + area.x;
             rects[i].y = rects[i].y * scale + area.y;
 
-            targets[i] = [rects[i].x, rects[i].y, scale];
+            slots.push([rects[i].x, rects[i].y, scale, clones[i]]);
         }
 
-        return [clones, targets];
+        return slots;
     }
     workspaceInjections['_calculateWindowTransformationsNatural'] = undefined;
 
-    /**
-     * _realPositionWindows:
-     * @flags:
-     *  INITIAL - this is the initial positioning of the windows.
-     *  ANIMATE - Indicates that we need animate changing position.
-     */
-    workspaceInjections['_realPositionWindows'] = Workspace.Workspace.prototype._realPositionWindows;
-    Workspace.Workspace.prototype._realPositionWindows = function(flags) {
-        if (this._repositionWindowsId > 0) {
-            Mainloop.source_remove(this._repositionWindowsId);
-            this._repositionWindowsId = 0;
-        }
-
-        let clones = this._windows.slice();
-        if (this._reservedSlot)
-            clones.push(this._reservedSlot);
-
-        let initialPositioning = flags & WindowPositionFlags.INITIAL;
-        let animate = flags & WindowPositionFlags.ANIMATE;
-
-        // Start the animations
-        let targets = [];
-        let scales = [];
-
-        [clones, targets] = this._calculateWindowTransformationsNatural(clones);
-
-        let currentWorkspace = global.screen.get_active_workspace();
-        let isOnCurrentWorkspace = this.metaWorkspace == null || this.metaWorkspace == currentWorkspace;
-
-        for (let i = 0; i < clones.length; i++) {
-            let clone = clones[i];
-            let [x, y , scale] = targets[i];
-            let metaWindow = clone.metaWindow;
-            let mainIndex = this._lookupIndex(metaWindow);
-            let overlay = this._windowOverlays[mainIndex];
-            clone.slotId = i;
-
-            // Positioning a window currently being dragged must be avoided;
-            // we'll just leave a blank spot in the layout for it.
-            if (clone.inDrag)
-                continue;
-
-            clone.slot = [x, y, clone.actor.width * scale, clone.actor.height * scale];
-
-            if (overlay && initialPositioning)
-                overlay.hide(initialPositioning);
-
-            if (animate && isOnCurrentWorkspace) {
-                if (!metaWindow.showing_on_its_workspace()) {
-                    /* Hidden windows should fade in and grow
-                     * therefore we need to resize them now so they
-                     * can be scaled up later */
-                    if (initialPositioning) {
-                        clone.actor.opacity = 0;
-                        clone.actor.scale_x = 0;
-                        clone.actor.scale_y = 0;
-                        clone.actor.x = x;
-                        clone.actor.y = y;
-                    }
-
-                    // Make the window slightly transparent to indicate it's hidden
-                    Tweener.addTween(clone.actor,
-                                     { opacity: 255,
-                                       time: Overview.ANIMATION_TIME,
-                                       transition: 'easeInQuad'
-                                     });
-                }
-
-                this._animateClone(clone, overlay, x, y, scale, initialPositioning);
-            } else {
-                clone.actor.set_position(x, y);
-                clone.actor.set_scale(scale, scale);
-                clone.overlay.relayout(false);
-                this._showWindowOverlay(clone, overlay, isOnCurrentWorkspace);
-            }
-        }
+    /// map gnome shell's computeAllWindowSlots() to our window placement function
+    workspaceInjections['_computeAllWindowSlots'] = Workspace.Workspace.prototype._computeAllWindowSlots;
+    Workspace.Workspace.prototype._computeAllWindowSlots = function(windows) {
+        return this._calculateWindowTransformationsNatural(windows);
     }
 
     /// position window titles on top of windows in overlay ////
@@ -417,21 +342,20 @@ function enable() {
                     0];
         };
 
-        winInjections['updatePositions'] = Workspace.WindowOverlay.prototype.updatePositions;
-        Workspace.WindowOverlay.prototype.updatePositions = function(cloneX, cloneY, cloneWidth, cloneHeight, animate) {
+        winInjections['relayout'] = Workspace.WindowOverlay.prototype.relayout;
+        Workspace.WindowOverlay.prototype.relayout = function(animate) {
             let button = this.closeButton;
             let title = this.title;
+            let border = this.border;
 
-            let settings = new Gio.Settings({ schema: BUTTON_LAYOUT_SCHEMA });
-            let layout = settings.get_string(BUTTON_LAYOUT_KEY);
-            let rtl = Clutter.get_default_text_direction() == Clutter.TextDirection.RTL;
+            Tweener.removeTweens(button);
+            Tweener.removeTweens(title);
+            Tweener.removeTweens(border);
 
-            let split = layout.split(":");
-            let side;
-            if (split[0].indexOf("close") > -1)
-                side = rtl ? St.Side.RIGHT : St.Side.LEFT;
-            else
-                side = rtl ? St.Side.LEFT : St.Side.RIGHT;
+            let [cloneX, cloneY, cloneWidth, cloneHeight] = this._windowClone.slot;
+
+            let layout = Meta.prefs_get_button_layout();
+            let side = layout.left_buttons.indexOf(Meta.ButtonFunction.CLOSE) > -1 ? St.Side.LEFT : St.Side.RIGHT;
 
             let buttonX;
             let buttonY = cloneY - (button.height - button._overlap);
@@ -445,11 +369,21 @@ function enable() {
             else
                 button.set_position(Math.floor(buttonX), Math.floor(buttonY));
 
-            if (!title.fullWidth)
-                title.fullWidth = title.width;
-            let titleWidth = Math.min(title.fullWidth, cloneWidth);
+            // Clutter.Actor.get_preferred_width() will return the fixed width if one
+            // is set, so we need to reset the width by calling set_width(-1), to forward
+            // the call down to StLabel.
+            // We also need to save and restore the current width, otherwise the animation
+            // starts from the wrong point.
+            let prevTitleWidth = title.width;
+            title.set_width(-1);
+            let [titleMinWidth, titleNatWidth] = title.get_preferred_width(-1);
+            let titleWidth = Math.max(titleMinWidth, Math.min(titleNatWidth, cloneWidth));
+            title.width = prevTitleWidth;
 
             let titleX = cloneX + (cloneWidth - titleWidth) / 2;
+
+            /// this is the actual difference to original gnome-shell:
+            //let titleY = cloneY + cloneHeight + title._spacing;
             let titleY = cloneY - title.height + title._spacing;
 
             if (animate)
@@ -458,6 +392,19 @@ function enable() {
                 title.width = titleWidth;
                 title.set_position(Math.floor(titleX), Math.floor(titleY));
             }
+
+            let borderX = cloneX - this.borderSize;
+            let borderY = cloneY - this.borderSize;
+            let borderWidth = cloneWidth + 2 * this.borderSize;
+            let borderHeight = cloneHeight + 2 * this.borderSize;
+
+            if (animate) {
+                this._animateOverlayActor(this.border, borderX, borderY,
+                                          borderWidth, borderHeight);
+            } else {
+                this.border.set_position(borderX, borderY);
+                this.border.set_size(borderWidth, borderHeight);
+            }
         };
     }
 }
